local _, Addon = ...
local Ability = Addon:NewModule('Ability')
Ability.__index = Ability


--[[ Constructor ]]--

function Ability:New(pet, index)
	local usable, level, cooldown, onCooldown	
	local id, _, icon = pet:GetAbilityInfo(index)
	local type = pet:GetAbilityType(index)

	if id then
		usable, cooldown = pet:GetAbilityState(index)
		onCooldown = cooldown and cooldown > 0
		usable = usable and not onCooldown
	else 
		id, level, name, icon = self:FindInJournal(pet, index)
	end
	
	local ability = {
		id = id,
		pet = pet,
		index = index,
		
		name = name,
		icon = icon,
		level = level,
		type = type,
		
		onCooldown = onCooldown,
		cooldown = cooldown,
		usable = usable
	}
	
	return setmetatable(ability, self)
end

function Ability:FindInJournal(pet, index)
	local ids, levels = C_PetJournal.GetPetAbilityList(pet:GetSpecie())
	local id = ids[index]

	return id, levels[index], C_PetJournal.GetPetAbilityInfo(id)
end


--[[ API ]]--

function Ability:GetIdentifiers()
	return self.pet.owner, self.pet.index, self.id
end

function Ability:GetIcon()
	return self.icon, self.usable
end

function Ability:GetCooldown()
	return self.cooldown, self.onCooldown
end

function Ability:GetModifierFor(target)
	return target:GetModifier(self.type)
end