local _, Addon = ...
local Map = CreateFrame('Frame')

local Journal = LibStub('LibPetJournal-2.0')
local JounralInfo = C_PetJournal

local NumberMatch = '(%d+%.?%d*)'
local SpotMatch = NumberMatch .. '|' .. NumberMatch

local BlipParent = WorldMapButton
local MapFrame = WorldMapDetailFrame
local Tooltip = WorldMapTooltip


--[[ Update ]]--

function Map:Startup()
	self:RegisterEvent('MINIMAP_UPDATE_TRACKING')
	self:SetScript('OnEvent', self.Update)
	self.blips = {}
	
	hooksecurefunc('WorldMapFrame_Update', function()
		self:Update()
	end)
end

function Map:Update()
	local width, height = MapFrame:GetSize()
	local creatures = self:GetCurrentCreatures()
	local index = 0
	
	if Addon:TrackingPets() then
		for specie, spots in pairs(creatures) do
			local type = select(3, JounralInfo.GetPetInfoBySpeciesID(specie))
			local icon = 'Interface/PetBattles/PetIcon-' .. PET_TYPE_SUFFIX[type]
	
			for _, spot in ipairs(spots) do
				index = index + 1
				
				local x, y = self:GetCoords(spot)
				local blip = self:GetBlip(index)
				blip:SetPoint('CENTER', MapFrame, 'TOPLEFT', x * width, -y * height)
				blip.icon:SetTexture(icon)
				blip.specie = specie
				blip:Show()
			end
		end
	end
	
	for i = index + 1, #self.blips do
		self.blips[i]:Hide()
	end
end


--[[ Blips ]]--

function Map:GetBlip(i)
	return self.blips[i] or self:CreateBlip(i)
end

function Map:CreateBlip(i)
	local blip = CreateFrame('Frame', nil, BlipParent, 'WorldMapUnitTemplate')
	blip.icon:SetTexCoord(0.79687500, 0.49218750, 0.50390625, 0.65625000)
	blip:SetScript('OnEnter', self.OnEnter)
	blip:SetScript('OnLeave', self.OnLeave)
	blip:SetSize(13, 13)
	
	self.blips[i] = blip
	return blip
end

function Map:OnEnter()
	local anchorLeft = self:GetCenter() > BlipParent:GetCenter()
	local name, icon, _,_, source = JounralInfo.GetPetInfoBySpeciesID(self.specie)
	
	local title = Map:ToString(icon) .. name
	local text = Map:KeepShort(source)
	
	WorldMapPOIFrame.allowBlobTooltip = false
	Tooltip:SetOwner(self, 'ANCHOR_' .. (anchorLeft and 'LEFT' or 'RIGHT'))
	Tooltip:SetText(title, 1,1,1)
	Tooltip:AddLine(text, 1,1,1, true)
	Tooltip:Show()
end

function Map:OnLeave()
	WorldMapPOIFrame.allowBlobTooltip = true
	Tooltip:Hide()
end


--[[ Text Formatting ]]--

function Map:ToString(icon)
	return '|T' .. icon .. ':20:20:0:0|t'
end

function Map:KeepShort(text)
	if not text:find('|n') and strlen(text) > 120 then
		return text:sub(0, 117) .. '...'
	end
	
	return text
end


--[[ Data ]]--

function Map:GetCurrentCreatures()
	return self:GetCreaturesIn(self:GetLocation())
end

function Map:GetCreaturesIn(zone, level)
	zone = PetTracker_Spots[zone]
	return zone and zone[level] or {}
end

function Map:GetLocation()
	return GetCurrentMapAreaID(), self:GetLevel()
end

function Map:GetLevel()
	local level = GetCurrentMapDungeonLevel()
	return DungeonUsesTerrainMap() and (level -1) or level
end

function Map:GetCoords(spot)
	local x, y = spot:match(SpotMatch)
	return tonumber(x) / 100, tonumber(y) / 100
end


Addon:NewModule('WorldMap', Map)